/*
 * Copyright (c) 2009-2015 Oliver Mueller [http://oliver-mueller.com]
 * 
 * This file is part of WOPR.
 * 
 * $Id: WoprLogger.cpp 69 2015-07-27 02:31:25Z oliver $
 * 
 * WOPR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <boost/date_time.hpp>
#include "WoprLogger.h"

namespace wopr
{

boost::mutex WoprLogger::cout_mutex_;

const char *WoprLogger::log_level_id_[4] = {
    "[error]",
    "[warning]",
    "[info]",
    "[debug]"
};

WoprLogger::WoprLogger(LogLevel level)
{
    log_level_ = level;
}

WoprLogger::~WoprLogger()
{
    WoprLogger::cout_mutex_.lock();
    std::cout << "["
            << boost::posix_time::to_iso_extended_string(boost::posix_time::second_clock::local_time())
            << "] ";
#if defined(__xlC__) || defined(__ibmxl__) || defined(__SUNPRO_CC) || defined(LEGACY_GCC)
    std::cout << std::setfill(' ') << std::setw(9) << std::left << log_level_id_[log_level_];
#else
    std::cout << std::setfill(' ') << std::setw(9) << std::left << log_level_id_[static_cast<int>(log_level_)];
#endif
    std::cout << " " << output_buffer_.rdbuf() << std::endl;
    WoprLogger::cout_mutex_.unlock();
}

} // namespace wopr
