/*
 * Copyright (c) 2009-2015 Oliver Mueller [http://oliver-mueller.com]
 * 
 * This file is part of WOPR.
 * 
 * $Id: WoprLogger.h 69 2015-07-27 02:31:25Z oliver $
 * 
 * WOPR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef WOPRLOGGER_H
#define	WOPRLOGGER_H

#include <iostream>
#include <iomanip>
#include <boost/thread.hpp>
#include "config.h"

#if defined(__xlC__) || defined(__ibmxl__) || defined(__SUNPRO_CC) || defined(LEGACY_GCC)
#define WOPR_LOG(x) ::wopr::WoprLogger(::wopr::x).get_stringstream()
#else
#define WOPR_LOG(x) ::wopr::WoprLogger(::wopr::LogLevel::x).get_stringstream()
#endif

namespace wopr
{

#if defined(__xlC__) || defined(__ibmxl__) || defined(__SUNPRO_CC) || defined(LEGACY_GCC)
typedef enum { kError = 0, kWarning = 1, kInfo = 2, kDebug = 3 } LogLevel;
#else
enum class LogLevel { kError = 0, kWarning = 1, kInfo = 2, kDebug = 3 };
#endif

class WoprLogger
{
public:
    explicit WoprLogger(LogLevel level);
    virtual ~WoprLogger();

    template <typename T>
    friend WoprLogger& operator<<(WoprLogger& logger, T const& t)
    {
        logger.output_buffer_ << t;
        return logger;
    }
    
    std::stringstream& get_stringstream();

private:
    std::stringstream output_buffer_;
    LogLevel log_level_;
    static const char *log_level_id_[4];

    static boost::mutex cout_mutex_;
};

inline std::stringstream& WoprLogger::get_stringstream()
{
    return output_buffer_;
}

} // namespace wopr

#endif	/* WOPRLOGGER_H */

