/*
 * Copyright (c) 2009-2015 Oliver Mueller [http://oliver-mueller.com]
 * 
 * This file is part of WOPR.
 * 
 * $Id: WoprServer.cpp 70 2015-07-29 13:07:47Z oliver $
 * 
 * WOPR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "WoprServer.h"

namespace wopr {

WoprServer::WoprServer(boost::asio::io_service& io_service,
                       unsigned short port,
                       std::string key_file,
                       std::string password,
                       std::string cert_file,
                       bool join,
                       bool ipv6,
                       bool multi_threading,
                       unsigned delay)
: io_service_(io_service),
acceptor_(io_service,
          boost::asio::ip::tcp::endpoint((ipv6 ? boost::asio::ip::tcp::v6()
                                         : boost::asio::ip::tcp::v4()), port)),
context_(io_service, boost::asio::ssl::context::sslv23),
password_(password)
{
    context_.set_options(boost::asio::ssl::context::default_workarounds
                         | boost::asio::ssl::context::no_sslv2
                         | boost::asio::ssl::context::single_dh_use);
    context_.set_password_callback(boost::bind(&WoprServer::get_password, this));
    context_.use_certificate_chain_file(cert_file.c_str());
    context_.use_private_key_file(key_file.c_str(),
                                  boost::asio::ssl::context::pem);

    join_ = join;
    client_number_ = 0;
    multi_threading_ = multi_threading;
    delay_ = delay;

    WoprSession* new_session = new WoprSession(io_service_, context_, client_number_++, join, multi_threading, delay);
    acceptor_.async_accept(new_session->socket(),
                           boost::bind(&WoprServer::handle_accept,
                                       this,
                                       new_session,
                                       boost::asio::placeholders::error)
                           );
}

WoprServer::~WoprServer()
{
}

void WoprServer::handle_accept(WoprSession* new_session,
                               const boost::system::error_code& error)
{
    if (!error)
    {
        new_session->start();
        new_session = new WoprSession(io_service_, context_, client_number_++, join_, multi_threading_, delay_);
        acceptor_.async_accept(new_session->socket(),
                               boost::bind(&WoprServer::handle_accept, this,
                                           new_session,
                                           boost::asio::placeholders::error));
    }
    else
    {
        delete new_session;
    }
}

}
