/*
 * Copyright (c) 2009-2015 Oliver Mueller [http://oliver-mueller.com]
 * 
 * This file is part of WOPR.
 * 
 * $Id: WoprServer.h 70 2015-07-29 13:07:47Z oliver $
 * 
 * WOPR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef WOPRSERVER_H
#define	WOPRSERVER_H

#include <string>
#include <boost/system/error_code.hpp>
#include <boost/thread.hpp>
#include <boost/asio.hpp>

#include "WoprSession.h"

namespace wopr {

class WoprServer
{
public:
    WoprServer(boost::asio::io_service& io_service, unsigned short port, std::string key_file, std::string password, std::string cert_file, bool join, bool ipv6, bool multi_threading, unsigned delay);
    virtual ~WoprServer();

    std::string get_password() const;

protected:
    void handle_accept(WoprSession* new_session, const boost::system::error_code& error);


private:
    boost::asio::io_service& io_service_;
    boost::asio::ip::tcp::acceptor acceptor_;
    boost::asio::ssl::context context_;
    std::string password_;
    bool join_;
    unsigned long long client_number_;
    bool multi_threading_;
    unsigned delay_;
};

inline std::string WoprServer::get_password() const
{
    return password_;
}

} // namespace wopr

#endif	/* WOPRSERVER_H */

